@@ -1,14 +1,14 @@
 <?php
-
-/* $Id: config.sample.inc.php 9689 2006-11-10 20:05:49Z nijel $ */
-// vim: expandtab sw=4 ts=4 sts=4:
-
+/* vim: set expandtab sw=4 ts=4 sts=4: */
 /**
  * phpMyAdmin sample configuration, you can use it as base for
- * manual configuration. For easier setup you can use scripts/setup.php
+ * manual configuration. For easier setup you can use setup/
  *
  * All directives are explained in Documentation.html and on phpMyAdmin
- * wiki <http://wiki.cihar.com>.
+ * wiki <http://wiki.phpmyadmin.net>.
+ *
+ * @version $Id: config.sample.inc.php 13111 2009-11-09 15:02:21Z lem9 $
+ * @package phpMyAdmin
  */
 
 /*
@@ -34,6 +34,14 @@ $cfg['Servers'][$i]['connect_type'] = 'tcp';
 $cfg['Servers'][$i]['compress'] = false;
 /* Select mysqli if your server has it */
 $cfg['Servers'][$i]['extension'] = 'mysql';
+$cfg['Servers'][$i]['AllowNoPassword'] = false;
+
+/* rajk - for blobstreaming */
+$cfg['Servers'][$i]['bs_garbage_threshold'] = 50;
+$cfg['Servers'][$i]['bs_repository_threshold'] = '32M';
+$cfg['Servers'][$i]['bs_temp_blob_timeout'] = 600;
+$cfg['Servers'][$i]['bs_temp_log_threshold'] = '32M';
+
 /* User for advanced features */
 // $cfg['Servers'][$i]['controluser'] = 'pma';
 // $cfg['Servers'][$i]['controlpass'] = 'pmapass';
@@ -47,6 +55,8 @@ $cfg['Servers'][$i]['extension'] = 'mysql';
 // $cfg['Servers'][$i]['column_info'] = 'pma_column_info';
 // $cfg['Servers'][$i]['history'] = 'pma_history';
 // $cfg['Servers'][$i]['designer_coords'] = 'pma_designer_coords';
+/* Contrib / Swekey authentication */
+// $cfg['Servers'][$i]['auth_swekey_config'] = '/etc/swekey-pma.conf';
 
 /*
  * End of servers configuration
