@@ -3,7 +3,8 @@
 /**
  * Set of functions used to build CSV dumps of tables
  *
- * @version $Id: excel.php 11326 2008-06-17 21:32:48Z lem9 $
+ * @package phpMyAdmin-Export-CSV
+ * @version $Id: excel.php 12494 2009-05-25 08:11:32Z helmo $
  */
 if (! defined('PHPMYADMIN')) {
     exit;
@@ -19,8 +20,16 @@ if (isset($plugin_list)) {
         'mime_type' => 'text/comma-separated-values',
         'options' => array(
             array('type' => 'text', 'name' => 'null', 'text' => 'strReplaceNULLBy'),
+            array('type' => 'bool', 'name' => 'removeCRLF', 'text' => 'strRemoveCRLF'),
             array('type' => 'bool', 'name' => 'columns', 'text' => 'strPutColNames'),
-            array('type' => 'select', 'name' => 'edition', 'values' => array('win' => 'Windows', 'mac' => 'Excel 2003 / Macintosh'), 'text' => 'strExcelEdition'),
+            array(
+                'type' => 'select', 
+                'name' => 'edition', 
+                'values' => array(
+                    'win' => 'Windows',
+                    'mac_excel2003' => 'Excel 2003 / Macintosh', 
+                    'mac_excel2008' => 'Excel 2008 / Macintosh'), 
+                'text' => 'strExcelEdition'),
             array('type' => 'hidden', 'name' => 'data'),
             ),
         'options_text' => 'strOptions',
