@@ -3,7 +3,8 @@
 /**
  * Produce a PDF report (export) from a query
  *
- * @version $Id: pdf.php 11326 2008-06-17 21:32:48Z lem9 $
+ * @package phpMyAdmin-Export-PDF
+ * @version $Id: pdf.php 11986 2008-11-24 11:05:40Z nijel $
  */
 if (! defined('PHPMYADMIN')) {
     exit;
@@ -35,14 +36,27 @@ if (isset($plugin_list)) {
 define('PMA_PDF_FONT', 'DejaVuSans');
 require_once './libraries/tcpdf/tcpdf.php';
 
-// Adapted from a LGPL script by Philip Clarke
-
+/**
+ * Adapted from a LGPL script by Philip Clarke
+ * @package phpMyAdmin-Export-PDF
+ */
 class PMA_PDF extends TCPDF
 {
     var $tablewidths;
     var $headerset;
     var $footerset;
 
+    // added because tcpdf for PHP 5 has a protected $buffer
+    public function getBuffer()
+    {
+        return $this->buffer;
+    }
+
+    public function getState()
+    {
+        return $this->state;
+    }
+
     // overloading of a tcpdf function:
     function _beginpage($orientation)
     {
@@ -236,7 +250,7 @@ class PMA_PDF extends TCPDF
             for ($i = 0; $i < $this->numFields; $i++){
                 $stringWidth = $this->getstringwidth($this->fields[$i]->name) + 6 ;
                 // save the real title's width
-                $titleWidth[$i] = $stringWidth; 
+                $titleWidth[$i] = $stringWidth;
                 $totalTitleWidth += $stringWidth;
 
                 // set any column titles less than the start width to the column title width
@@ -281,8 +295,8 @@ class PMA_PDF extends TCPDF
                 }
             }
 
-            // loop through the data, any column whose contents is bigger
-            // than the col size is resized
+            // loop through the data; any column whose contents
+            // is greater than the column size is resized
             /**
               * @todo force here a LIMIT to avoid reading all rows
               */
@@ -453,10 +467,10 @@ function PMA_exportData($db, $table, $crlf, $error_url, $sql_query)
     $pdf->mysql_report($sql_query, $attr);
 
     // instead of $pdf->Output():
-    if ($pdf->state < 3) {
+    if ($pdf->getState() < 3) {
         $pdf->Close();
     }
-    if (!PMA_exportOutputHandler($pdf->buffer)) {
+    if (!PMA_exportOutputHandler($pdf->getBuffer())) {
         return FALSE;
     }
 
