@@ -19,12 +19,17 @@
  * It's easier to use only uppercase for proper sorting. In case of
  * doubt, use the DEBUG code after this function's definition.
  *
- * @version$Id: sqlparser.data.php 11326 2008-06-17 21:32:48Z lem9 $
+ * @version $Id: sqlparser.data.php 13068 2009-10-24 12:25:48Z lem9 $
+ * @package phpMyAdmin
  */
 if (! defined('PHPMYADMIN')) {
     exit;
 }
 
+if (! isset($GLOBALS['sql_delimiter'])) {
+    $GLOBALS['sql_delimiter'] = ';';
+}
+
 /**
  * @global array MySQL function names
  */
@@ -366,6 +371,7 @@ $PMA_SQPdata_column_attrib = array (
     'INNOBASE',         // Engine alias InnoDB
     'INNODB',           // Engine InnoDB
     'ISAM',             // Engine
+    'MARIA',            // Engine
     'MEMORY',           // Engine alias HEAP, but preferred
     'MERGE',            // Engine
     'MRG_ISAM',         // Engine
@@ -386,7 +392,7 @@ $PMA_SQPdata_column_attrib = array (
  *
  * @global integer MySQL attributes count
  */
-$PMA_SQPdata_column_attrib_cnt = 29;
+$PMA_SQPdata_column_attrib_cnt = 30;
 
 /**
  * words that are reserved by MySQL and may not be used as identifiers without quotes
@@ -436,6 +442,7 @@ $PMA_SQPdata_reserved_word = array (
     'COMPRESSED',
     'CONCURRENT',
     'CONSTRAINT',
+    'CONTAINS',
     'CONVERT',
     'CREATE',
     'CROSS',
@@ -453,6 +460,7 @@ $PMA_SQPdata_reserved_word = array (
     'DELETE',
     'DESC',
     'DESCRIBE',
+    'DETERMINISTIC',
     'DISTINCT',
     'DISTINCTROW',
     'DIV',
@@ -512,6 +520,7 @@ $PMA_SQPdata_reserved_word = array (
     'INSERT_METHOD',
     'INTERVAL',
     'INTO',
+    'INVOKER',
     'IS',
     'ISOLATION',
     'JOIN',
@@ -532,6 +541,7 @@ $PMA_SQPdata_reserved_word = array (
     'LOCKS',
     'LOGS',
     'LOW_PRIORITY',
+    'MARIA',                // 5.1 ?
     'MASTER',
     'MASTER_CONNECT_RETRY',
     'MASTER_HOST',
@@ -573,7 +583,10 @@ $PMA_SQPdata_reserved_word = array (
     'OUTER',
     'OUTFILE',
     'PACK_KEYS',
+    'PAGE',                 // 5.1-maria ?
     'PARTIAL',
+    'PARTITION',            // 5.1
+    'PARTITIONS',           // 5.1
     'PASSWORD',
     'PRIMARY',
     'PRIVILEGES',
@@ -611,6 +624,7 @@ $PMA_SQPdata_reserved_word = array (
     'ROWS',
     'ROW_FORMAT',
     'SECOND',
+    'SECURITY',
     'SELECT',
     'SEPARATOR',
     'SERIALIZABLE',
@@ -621,6 +635,7 @@ $PMA_SQPdata_reserved_word = array (
     'SLAVE',
     'SONAME',
     'SOUNDS',                   // string-functions.html
+    'SQL',
     'SQL_AUTO_IS_NULL',
     'SQL_BIG_RESULT',
     'SQL_BIG_SELECTS',
@@ -656,6 +671,7 @@ $PMA_SQPdata_reserved_word = array (
     'THEN',
     'TO',
     'TRAILING',
+    'TRANSACTIONAL',    // 5.1 ?
     'TRUNCATE',
     'TYPE',
     'TYPES',
@@ -683,9 +699,9 @@ $PMA_SQPdata_reserved_word = array (
  *
  * @global integer MySQL reserved words count
  */
-$PMA_SQPdata_reserved_word_cnt = 279;
+$PMA_SQPdata_reserved_word_cnt = 289;
 /**
- * The previous array must be sorted so that the binary search work. 
+ * The previous array must be sorted so that the binary search work.
  * Sometimes a word is not added in the correct order, so
  * this debugging code shows the problem. The same should be
  * done for all arrays.
@@ -701,6 +717,7 @@ echo '<pre>';
 print_r($PMA_SQPdata_reserved_word);
 echo '</pre>';
 */
+
 /**
  * words forbidden to be used as column or table name wihtout quotes
  * as seen in http://dev.mysql.com/doc/mysql/en/reserved-words.html
@@ -1250,7 +1267,7 @@ $PMA_SQPdata_column_type = array (
     'POINT',                // spatial
     'POLYGON',              // spatial
     'REAL',
-    'SERIAL',               // alsias for BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY
+    'SERIAL',               // alias for BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY
     'SET',
     'SMALLINT',
     'TEXT',
