@@ -1,9 +1,15 @@
 <?php
-/* $Id: image_png__inline.inc.php 6998 2005-05-22 17:11:10Z lem9 $ */
-// vim: expandtab sw=4 ts=4 sts=4:
+/* vim: set expandtab sw=4 ts=4 sts=4: */
+/**
+ * @package phpMyAdmin-Transformation
+ * @version $Id: image_png__inline.inc.php 11973 2008-11-24 09:30:37Z nijel $
+ */
 
+/**
+ *
+ */
 function PMA_transformation_image_png__inline($buffer, $options = array(), $meta = '') {
-    require_once('./libraries/transformations/global.inc.php');
+    require_once './libraries/transformations/global.inc.php';
 
     if (PMA_IS_GD2) {
         $transform_options = array ('string' => '<a href="transformation_wrapper.php' . $options['wrapper_link'] . '" target="_blank"><img src="transformation_wrapper.php' . $options['wrapper_link'] . '&amp;resize=png&amp;newWidth=' . (isset($options[0]) ? $options[0] : '100') . '&amp;newHeight=' . (isset($options[1]) ? $options[1] : 100) . '" alt="[__BUFFER__]" border="0" /></a>');
